@@ -1,6 +1,7 @@
 import datetime
 from django.core.urlresolvers import reverse
 from django.db.models.sql.constants import QUERY_TERMS
+from django.utils.encoding import force_unicode
 from tastypie import fields
 
 from .utils import trailing_slash_or_none, urljoin_forced
@@ -26,11 +27,41 @@ class ResourceSwaggerMapping(object):
     https://github.com/wordnik/swagger-core/wiki/API-Declaration
     """
     WRITE_ACTION_IGNORED_FIELDS = ['id', 'resource_uri',]
+
+    # Default summary strings for operations
+    OPERATION_SUMMARIES = {
+        'get-detail': "Retrieve a single %s by ID",
+        'get-list': "Retrieve a list of %s",
+        'post-list': "Create a new %s",
+        'put-detail': "Update an existing %s",
+        'delete-detail': "Delete an existing %s",
+    }
+
     def __init__(self, resource):
         self.resource = resource
         self.resource_name = self.resource._meta.resource_name
         self.schema = self.resource.build_schema()
 
+    def get_resource_verbose_name(self, plural=False):
+        qs = self.resource._meta.queryset
+        if qs is not None and hasattr(qs, 'model'):
+            meta = qs.model._meta
+            verbose_name = plural and meta.verbose_name_plural or meta.verbose_name
+            return verbose_name.lower()
+        return self.resource_name
+
+    def get_operation_summary(self, detail=True, method='get'):
+        """
+        Get a basic summary string for a single operation
+        """
+        key = '%s-%s' % (method.lower(), detail and 'detail' or 'list')
+        plural = not detail and method is 'get'
+        verbose_name = self.get_resource_verbose_name(plural=plural)
+        summary = self.OPERATION_SUMMARIES.get(key, '')
+        if summary:
+            return summary % verbose_name
+        return ''
+
     def get_resource_base_uri(self):
         """
         Use Resource.get_resource_list_uri (or Resource.get_resource_uri, depending on version of tastypie)
@@ -69,7 +100,7 @@ class ResourceSwaggerMapping(object):
                     name=name,
                     dataType=field['type'],
                     required=not field['blank'],
-                    description=unicode(field['help_text']),
+                    description=force_unicode(field['help_text']),
                 ))
         return parameters
 
@@ -113,7 +144,7 @@ class ResourceSwaggerMapping(object):
                     name=name,
                     dataType=type,
                     required=False,
-                    description=unicode(desc),
+                    description=force_unicode(desc),
                 ))
         if 'filtering' in self.schema and method.upper() == 'GET':
             for name, field in self.schema['filtering'].items():
@@ -152,12 +183,20 @@ class ResourceSwaggerMapping(object):
                     schema_field = self.schema['fields'][name]
                     for query in field:
                         if query == 'exact':
+                            description = force_unicode(schema_field['help_text'])
+                            dataType = schema_field['type']
+                            # Use a better description for related models with exact filter
+                            if dataType == 'related':
+                                # Assume that related pk is an integer
+                                # TODO if youre not using integer ID for pk then we need to look this up somehow
+                                dataType = 'integer'
+                                description = 'ID of related resource'
                             parameters.append(self.build_parameter(
                                 paramType="query",
                                 name="%s%s" % (prefix, name),
-                                dataType=schema_field['type'],
+                                dataType=dataType,
                                 required= False,
-                                description=unicode(schema_field['help_text']),
+                                description=description,
                             ))
                         else:
                             parameters.append(self.build_parameter(
@@ -165,7 +204,7 @@ class ResourceSwaggerMapping(object):
                                 name="%s%s__%s" % (prefix, name, query),
                                 dataType=schema_field['type'],
                                 required= False,
-                                description=unicode(schema_field['help_text']),
+                                description=force_unicode(schema_field['help_text']),
                             ))
 
         return parameters
@@ -177,25 +216,32 @@ class ResourceSwaggerMapping(object):
             required= True
         )
 
+    def _detail_uri_name(self):
+        # For compatibility with TastyPie 0.9.11, which doesn't define a
+        # detail_uri_name by default.
+        detail_uri_name = getattr(self.resource._meta, "detail_uri_name", "pk")
+        return detail_uri_name == "pk" and "id" or detail_uri_name
+
     def build_parameters_from_extra_action(self, method, fields):
         parameters = []
         if method.upper() == 'GET':
-            parameters.append(self.build_parameter(paramType='path', name='%s' % self.resource._meta.detail_uri_name if self.resource._meta.detail_uri_name != 'pk' else "id", dataType='int', description='ID of resource'))
+            parameters.append(self.build_parameter(paramType='path', name=self._detail_uri_name(), dataType='int', description='ID of resource'))
         for name, field in fields.items():
             parameters.append(self.build_parameter(
                 paramType="query",
                 name=name,
                 dataType=field['type'],
                 required=field['required'],
-                description=unicode(field['description']),
+                description=force_unicode(field['description']),
             ))
 
         return parameters
 
     def build_detail_operation(self, method='get'):
         operation = {
+            'summary': self.get_operation_summary(detail=True, method=method),
             'httpMethod': method.upper(),
-            'parameters': [self.build_parameter(paramType='path', name='%s' % self.resource._meta.detail_uri_name if self.resource._meta.detail_uri_name != 'pk' else "id", dataType='int', description='ID of resource')],
+            'parameters': [self.build_parameter(paramType='path', name=self._detail_uri_name(), dataType='int', description='ID of resource')],
             'responseClass': self.resource_name,
             'nickname': '%s-detail' % self.resource_name,
             'notes': self.resource.__doc__,
@@ -204,6 +250,7 @@ class ResourceSwaggerMapping(object):
 
     def build_list_operation(self, method='get'):
         return {
+            'summary': self.get_operation_summary(detail=False, method=method),
             'httpMethod': method.upper(),
             'parameters': self.build_parameters_for_list(method=method),
             'responseClass': 'ListView' if method.upper() == 'GET' else self.resource_name,
@@ -217,11 +264,11 @@ class ResourceSwaggerMapping(object):
             'parameters': self.build_parameters_from_extra_action(method=extra_action.get('http_method'), fields=extra_action.get('fields')),
             'responseClass': 'Object', #TODO this should be extended to allow the creation of a custom object.
             'nickname': extra_action['name'],
-            }
+        }
 
     def build_detail_api(self):
         detail_api = {
-            'path': urljoin_forced(self.get_resource_base_uri(), '{%s}%s' % (self.resource._meta.detail_uri_name if self.resource._meta.detail_uri_name != 'pk' else "id", trailing_slash_or_none())),
+            'path': urljoin_forced(self.get_resource_base_uri(), '{%s}%s' % (self._detail_uri_name(), trailing_slash_or_none())),
             'operations': [],
         }
 
@@ -257,7 +304,7 @@ class ResourceSwaggerMapping(object):
     def build_extra_apis(self):
         extra_apis = []
         if hasattr(self.resource._meta, 'extra_actions'):
-            identifier = self.resource._meta.detail_uri_name if self.resource._meta.detail_uri_name != 'pk' else 'id'
+            identifier = self._detail_uri_name()
             for extra_action in self.resource._meta.extra_actions:
                 extra_api = {
                     'path': "%s{%s}/%s/" % (self.get_resource_base_uri(), identifier , extra_action.get('name')),
@@ -268,16 +315,13 @@ class ResourceSwaggerMapping(object):
                 extra_apis.append(extra_api)
         return extra_apis
 
-
-
     def build_apis(self):
         apis = [self.build_list_api(), self.build_detail_api()]
         apis.extend(self.build_extra_apis())
         return apis
 
     def build_property(self, name, type, description=""):
-
-        property = {
+        prop = {
             name: {
                 'type': type,
                 'description': description,
@@ -285,9 +329,9 @@ class ResourceSwaggerMapping(object):
         }
 
         if type == 'List':
-            property[name]['items'] = {'$ref': name}
+            prop[name]['items'] = {'$ref': name}
 
-        return property
+        return prop
 
     def build_properties_from_fields(self, method='get'):
         properties = {}
@@ -308,7 +352,9 @@ class ResourceSwaggerMapping(object):
             properties.update(self.build_property(
                     name,
                     field.get('type'),
-                    field.get('help_text')
+                    # note: 'help_text' is a Django proxy which must be wrapped
+                    # in unicode *specifically* to get the actual help text.
+                    force_unicode(field.get('help_text', '')),
                 )
             )
         return properties
@@ -423,3 +469,4 @@ class ResourceSwaggerMapping(object):
         return models
 
 
+
