@@ -56,8 +56,9 @@ class JSONView(TemplateView):
         """
         Returns a response with a template rendered with the given context.
         """
-        if 'params' in context:
-            del context['params']
+        for k in ['params','view']:
+            if k in context:
+                del context[k]
         return self.response_class(
             json.dumps(context),
             content_type='application/json',
